@@ -1,21 +1,9 @@
-/*****************************************************************************
- * Copyright 2010 Miquel Canes Gonzalez <miquelcanes@gmail.com>              *
- * Copyright 2011-2014 Sven Brauch <svenbrauch@googlemail.com>               *
- *                                                                           *
- * This program is free software; you can redistribute it and/or             *
- * modify it under the terms of the GNU General Public License as            *
- * published by the Free Software Foundation; either version 2 of            *
- * the License, or (at your option) any later version.                       *
- *                                                                           *
- * This program is distributed in the hope that it will be useful,           *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
- * GNU General Public License for more details.                              *
- *                                                                           *
- * You should have received a copy of the GNU General Public License         *
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.     *
- *****************************************************************************
- */
+/*
+    SPDX-FileCopyrightText: 2010 Miquel Canes Gonzalez <miquelcanes@gmail.com>
+    SPDX-FileCopyrightText: 2011-2014 Sven Brauch <svenbrauch@googlemail.com>
+
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #ifndef EXPRESSIONVISITOR_H
 #define EXPRESSIONVISITOR_H
@@ -117,13 +105,14 @@ public:
     static TypePtr<T> typeObjectForIntegralType(const QString& typeDescriptor) {
         auto context = Helper::getDocumentationFileContext();
         if ( ! context ) {
-            AbstractType::Ptr null;
-            return null.cast<T>();
+            return {};
         }
         auto decls = context->findDeclarations(QualifiedIdentifier(typeDescriptor));
         auto decl = decls.isEmpty() ? nullptr : dynamic_cast<Declaration*>(decls.first());
-        auto type = decl ? decl->abstractType() : AbstractType::Ptr();
-        return type.cast<T>();
+        if ( ! decl ) {
+            return {};
+        }
+        return decl->abstractType().dynamicCast<T>();
     }
 
 private:
